/*
 *
 *	Class(ses):	AddressingPPE
 *	Revision:	$Id:$
 *	Author:	 Reinhardt Karnapke
 *
 *	Description:	An example of a ppe that uses Addresses
 *			
 *	Note:
 *			
 *
 *	(c) Reinhardt Karnapke, BTU Cottbus 2005
 */

#ifndef copra_CompressPPE_h
#define copra_CompressPPE_h



#include "copra/pps/RxMac.h"
#include "copra/pps/TxMac.h"
#include "copra/pps/RxCRC.h"
#include "copra/pps/TxCRC.h"
#include "copra/pps/Compression.h"

#include "copra/pps/RxDeadlineWrapper.h"
#include "copra/pps/TxDeadlineWrapper.h"
#include "copra/pps/AddressingStage.h"

#include "adt/Pool.h"
#include "copra/pps/NetBuffer.h"
#include "adt/GenericFiFo.h"
#include "copra/pps/Packet.h"
#include "conf.h"

namespace copra {

using adt::Pool;
using adt::GenericFiFo;


/**
 *	
 *	@author Reinhardt Karnapke
 *	@version $Id$
 */

class CompressPPE : public AddressingStage<NetAddress, NetBuffer> , public RxStage<NetBuffer> {
public: 

	

	/**
	*	Constructor. Here all parts of the ppe are assembled.
	*/
	CompressPPE()
	{

		// connect pps "up" : receiving way
		rxMac.rxConnect(&rxCRC);
		rxCRC.rxConnect(&compression);
		compression.rxConnect(this);
		
		// connect pps "down" : sending way
		this->txConnect(&compression);
		compression.txConnect(&txCRC);
		txCRC.txConnect(&txMac);

	}
	
	/**
	* Empty Destructor
	*/
	virtual ~CompressPPE()
	{
	}
	
	/**
	*	
	*/
	virtual void deliver(const NetAddress& n, NetBuffer* stack)
	{
		stack->push(n);
		txForward(stack);
	}
	
	void setID(NetAddress n)
	{
		id=n;
	}
	
	void setBroadcastID(NetAddress n)
	{
		broadcastID=n;
	}
	
	virtual void accept(NetBuffer* stack)
	{
		NetAddress m;
		stack->pop(m);
		if((m==id) || (m==broadcastID))
		{
			rxForward(stack);
		}
		else
		{
			stack->downRef();
		}
	}


	
	// the elements of the ppe
	RxMac rxMac;
	TxMac txMac;
	RxCRC rxCRC;
	TxCRC txCRC;
	Compression compression;
	NetAddress id;
	NetAddress broadcastID;
	
	
};

} //namespace copra
#endif
