/*
 *
 *	Class(ses):	TxCompression
 *	Revision:	$Id:$
 *	Author:	 Steffen Schulze
 *
 *	Description:	the downway of the Compression part of the pps
 *			
 *	Note:		
 *			
 *
 *	(c) Steffen Schulze, BTU Cottbus 2005
 */

#ifndef copra_Compression_h
#define copra_Compression_h

#include "copra/pps/TxStage.h"
#include "copra/pps/RxStage.h"
#include "copra/pps/NetBuffer.h"


#include "reflex/sinks/Fifo.h"
#include "reflex/sinks/Event.h"
#include "reflex/sinks/Sink.h"
#include "reflex/scheduling/ActivityFunctor.h"

#include "copra/pps/compress/CompressConfig.h"

#ifdef COMPRESS_DESIGNHAZARD
	#include "adt/GenericPool.h"
	using adt::GenericPool;
#endif

//using reflex::Activity;
#ifdef COMPRESS_TIMEOUT_ON
	#include "reflex/sinks/Sink.h"
	using reflex::Sink0;

	#include "reflex/timer/OneShotTimer.h"
	using reflex::OneShotTimer;
	
	#include "reflex/System.h"
	using reflex::getReflex;
#endif

#include "reflex/System.h"


namespace copra {

using reflex::Event;
using reflex::ActivityFunctor;
	
	
/**
 *	
 *	@author Steffen Schulze
 *	@version $Id$
 */

class Compression : 
	public TxStage<NetBuffer>,
	public RxStage<NetBuffer>
	#ifdef COMPRESS_TIMEOUT_ON
	, public Sink0
	#endif
{
	public:
		#ifdef COMPRESS_DESIGNHAZARD
		Compression(GenericPool* _pool) : 
			pool(_pool)
		#else
		Compression() :
		#endif
			#ifdef COMPRESS_MERGE_ON
			,funcFlush(100,100, *this)
			,eventFlush(&funcFlush)
			,mergedStack(mergedData, COMPRESS_BUFFER_MAXSIZE)
			,isMerging(false)
			#ifdef COMPRESS_TIMEOUT_ON
			,timer(*this)
			#endif
		#endif
		{
		}

		/**
		*	called by upper pps Element 
		*	to send a message which lies on the protocolstack
		*/
		virtual void deliver(NetBuffer* stack);
		virtual void accept(NetBuffer* stack);
		
	protected:	
		/**
		*	compression and decompression
		*/
		UINTMAX		process(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize, bool encode);
		/**
		*	merges the second stack on the first
		*/
		void		appendStack(Stack* dest, Stack* src);
	
	private:
		/**
		*	creates a new packet with inputbuffer as data
		*/
		NetBuffer*	createNetBuffer(BYTE* inputBuffer, UINTMAX inputBufferSize);
		
	#ifdef COMPRESS_DESIGNHAZARD
	private:
		GenericPool* pool;
	#endif
	

	#ifdef COMPRESS_MERGE_ON
	protected:
		/**
		*	merge and unmerging of packets
		*/
		NetBuffer*		merge(NetBuffer* newStack);
		NetBuffer*		unmerge(NetBuffer* stack);
	public:
		/**
		*	flushes the merged packet down the protocolstack
		*/
		void			flush();
	
	protected:
		ActivityFunctor<Compression, &Compression::flush>				funcFlush;
	
	public:
		Event eventFlush;
	
	protected:
		/**
		*	holds the uncompressed merged packets
		*/
		BYTE			mergedData[COMPRESS_BUFFER_MAXSIZE];
		Stack			mergedStack;
		
		/**
		*	a packet ready to flush
		*/
		NetBuffer*		mergedNetBuffer;
		bool			isMerging;
		
		#ifdef COMPRESS_TIMEOUT_ON
	public:
		virtual void		notify();
		OneShotTimer		timer;
		#endif
		
	#endif//merge

	#ifdef COMPRESS_DEBUG
	private:
		BYTE			countMerged;
	#endif


};

} //namespace copra
#endif
