/*
 *
 *	Class(ses):	CompressConfig
 *	Revision:	$Id:$
 *	Author:	 Steffen Schulze
 *
 *	Description:	the configuration and definition part for the compression stage
 *			
 *	Note:		
 *			
 *
 *	(c) Steffen Schulze, BTU Cottbus 2005
 */

#ifndef copra_CompressConfig_h
#define copra_CompressConfig_h

#include "compress/config.h"
#include "compress/CompressorCodec.h"

#include "conf.h"

//////////////////////
// Flags
/**
 *	Flag for merging of packets
 */
#define COMPRESS_MERGE_ON

#ifdef COMPRESS_MERGE_ON
	/**
	 *	Flag for flushtimout (see variable for timing)
	 */
	#define COMPRESS_TIMEOUT_ON
#endif


/**
 *	Flag for compression of packets (see compressionstack for config)
 */
#define COMPRESS_COMPRESS_ON

#define COMPRESS_DEBUG

#define COMPRESS_DESIGNHAZARD


//////////////////////
// Variables
enum
{
	/**
	 *	maximal size of a packet
	 */
	COMPRESS_BUFFER_PACKETSIZE	= (PacketPayload - 20),

	/**
	 *	maximal size of buffer for uncompressed merged packets
	 */
	COMPRESS_BUFFER_MAXSIZE		= (COMPRESS_BUFFER_PACKETSIZE * 2),

	/**
	 *	timeout for merging
	 */
	COMPRESS_TIMEOUT_FLUSH		= 4
};


//////////////////////
// Compressionstack
#ifdef COMPRESS_COMPRESS_ON
	//////////////////////
	// Compression inlcudes
	/**
	 *	uncomment includes for needed codecs
	 */
	//#include "compress/rle/RleCodec.h"
	#include "compress/rle2/Rle2Codec.h"
	//#include "compress/mtf/MtfCodec.h"
	//#include "compress/bwt/BwtCodec.h"
	//#include "compress/bwt2/Bwt2Codec.h"
	//#include "compress/huffman/HuffmanCodec.h"
	#include "compress/lzw/LzwCodec.h"
	
	
	//////////////////////
	// Config for compression
	/**
	 *	set count of codecs
	 */
	#define COMPRESS_CODEC_COUNT 2
	
	/**
	 *	change for codecstack (insert or delete codec, if codeccount differs)
	 */
	#define COMPRESS_CODEC_CODECS \
	Rle2Codec	codec0; \
	LzwCodec	codec1; \
	CompressorCodec* codecs[COMPRESS_CODEC_COUNT]; \
	\
	codecs[0]	= &codec0;\
	codecs[1]	= &codec1;
	
#endif //COMPRESS_COMPRESS_ON


	
//////////////////////
// Debugflags
#ifdef COMPRESS_DEBUG
	#include "adt/Debug.h"
	//#define COMPRESS_DEBUG_DETAIL
	#ifdef COMPRESS_DEBUG_DETAIL
		#include <iostream>
		using std::cerr;
		using std::endl;
	#endif

#endif


//////////////////////
// Debugcodes
	
#include "emulator/NetworkNode.h"
extern NetworkNode* activeNode;

#define COMPRESS_ACTIVENODE (activeNode->getID() * 1000)
#define COMPRESS_EVENTOFFSET 100000
	
enum
{
EVENT_COMPRESS_APPSTART		= 91 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_APPFINISH	= 92 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_APPWAIT		= 93 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_APPACCSTART	= 94 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_APPACCFINISH	= 95 * COMPRESS_EVENTOFFSET,

EVENT_COMPRESS_TXSTART		= 11 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_TXFINISH		= 12 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_MERGE		= 13 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_MERGEFLUSH	= 14 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_TXFORWARD	= 15 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_MERGESUCC	= 16 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_MERGESUCU	= 17 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_MERGELOWBUF	= 18 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_MERGEFULL	= 19 * COMPRESS_EVENTOFFSET,


EVENT_COMPRESS_RXSTART		= 21 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_RXFINISH		= 22 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_UNMERGE		= 23 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_UNMERGEUC	= 24 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_UNMERGEFLUSH	= 25 * COMPRESS_EVENTOFFSET,

EVENT_COMPRESS_TXERROR		= 31 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_RXERROR		= 32 * COMPRESS_EVENTOFFSET,

EVENT_COMPRESS_TIMERRESET	= 41 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_TIMERSTOP	= 42 * COMPRESS_EVENTOFFSET,
EVENT_COMPRESS_TIMERFLUSH	= 43 * COMPRESS_EVENTOFFSET,

EVENT_COMPRESS_RUN			= 50 * COMPRESS_EVENTOFFSET
};

#endif
