/*
 *
 *	Class(ses):	Compression
 *	Revision:	$Id:$
 *	Author:	 Steffen Schulze
 *
 *	Description:	The merged version of the Compression Rx and Tx paths of the pps
 *			
 *	Note:		
 *			
 *
 *	(c) Steffen Schulze, BTU Cottbus 2005
 */

#include "copra/pps/Compression.h"
#include "copra/pps/Packet.h"


namespace copra {


void Compression::deliver(NetBuffer* stack)
{
	stack->upRef();
	
	if(stack->getRefCount() == 1)
	{
		stack->downRef();
		return;
	}
	
	{//DEBUG
		#ifdef COMPRESS_DEBUG
		debug.inform(EVENT_COMPRESS_TXSTART + stack->used() + COMPRESS_ACTIVENODE, "> TxCompression::deliver()");
		#endif
	}
	

	#ifdef COMPRESS_MERGE_ON
		merge(stack);
	#else
		{//clear stack
		
		//much buffering because negative growing stack
		BYTE	dummy[COMPRESS_BUFFER_MAXSIZE];
		BYTE*	cacheBuffer = dummy;
		BYTE	cacheBufferSize;
		if(cacheBufferSize = process((BYTE*)(stack->tos()), stack->used(), cacheBuffer, COMPRESS_BUFFER_PACKETSIZE, true))
		{
			stack->downRef();
			stack->downRef();
			stack = createNetBuffer(cacheBuffer, cacheBufferSize);
			stack->upRef();
		}
		
	
		{//DEBUG
			#ifdef COMPRESS_DEBUG
			debug.inform(EVENT_COMPRESS_TXFORWARD + COMPRESS_ACTIVENODE + stack->used(), "< TxCompression::deliver() - FORWARD");
			#endif
		}
		
		}//clear stack
		txForward(stack);
	#endif
	stack->downRef();
}

void Compression::accept(NetBuffer* stack)
{
	stack->upRef();
	
	if(stack->getRefCount() == 1)
	{
		stack->downRef();
		return;
	}
	
	{//DEBUG
		#ifdef COMPRESS_DEBUG
		debug.inform(EVENT_COMPRESS_RXSTART + stack->used() + COMPRESS_ACTIVENODE, "> RxCompression::accept()");
		#endif
	}

	#ifdef COMPRESS_MERGE_ON
		unmerge(stack);
	#else
		{//clear stack
		BYTE	dummy[COMPRESS_BUFFER_MAXSIZE];
		BYTE*	cacheBuffer = dummy;
		BYTE	cacheBufferSize;
		if(cacheBufferSize = process((BYTE*)(stack->tos()), stack->used(), cacheBuffer, COMPRESS_BUFFER_MAXSIZE , false))
		{
			stack->downRef();
			stack->downRef();
			stack = createNetBuffer(cacheBuffer, cacheBufferSize);
			stack->upRef();
		}
		
		{//DEBUG
			#ifdef COMPRESS_DEBUG
	   		debug.inform(EVENT_COMPRESS_RXFINISH + stack->used() + COMPRESS_ACTIVENODE, "< RxCompression::accept()");
			#endif
		}
		}//clear stack		
		rxForward(stack);
	#endif

	stack->downRef();
}

} //namespace
