#ifndef MTFCODEC_H
#define MTFCODEC_H

/*
 * REFLEX - Real-time Event FLow EXecutive
 *
 * A lightweight operating system for deeply embedded systems.
 *
 *
 * Class(ses): MtfCodec
 * Revision: $Id:$
 * Author: Steffen Schulze
 *
 * Description: Codec that implements Move-To-Front Algorithm
 *
 * (c) Steffen Schulze, BTU Cottbus 2005
 */

#include "compress/CompressorCodec.h"

/**
 * Implementation of the Move-To-Front-Algorithm.
 * Starts with a normal alphabet. Each time a charactervalue appears
 * its index is written to output and the charactervalue in the alphabets
 * will be moved to front.
 *
 * @author Steffen Schulze
 * @version $Id$
 */
class MtfCodec : public CompressorCodec
{
	public:

		MtfCodec()
		{
			#ifdef TEXTOUTPUT
			textOutputMode = 1;
			#endif
		}
				
		/**
		 * Encode-method
		 *
		 * Implements the virtual method encode of the baseclass CompressionCodec
		 * @param inputBuffer is pointer to inputdata
		 * @param inputBufferSize is lenght of inputBuffer
		 * @param outputBuffer is pointer to memory for outputdata
		 * @param outputBufferSize is maximal lenght of outputdata
		 * @return length of outputBuffer
		 */
		virtual	UINTMAX encode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize);
		
		/**
		 * Decode-method
		 *
		 * Implements the virtual method decode of the baseclass CompressionCodec
		 * @param inputBuffer is pointer to inputdata
		 * @param inputBufferSize is lenght of inputBuffer
		 * @param outputBuffer is pointer to memory for outputdata
		 * @param outputBufferSize is maximal lenght of outputdata
		 * @return length of outputBuffer
		 */
		virtual	UINTMAX decode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize);
		
	protected:
		#ifdef TEXTOUTPUT
		int textOutputMode;
		#endif

};

#endif
