#ifndef RLE2CODEC_H
#define RLE2CODEC_H

/*
 * REFLEX - Real-time Event FLow EXecutive
 *
 * A lightweight operating system for deeply embedded systems.
 *
 *
 * Class(ses): RleCodec
 * Revision: $Id:$
 * Author: Steffen Schulze
 *
 * Description: Codec that implements an advanced Run-Length-Encoding
 *
 * (c) Steffen Schulze, BTU Cottbus 2005
 */

#include "compress/CompressorCodec.h"

/**
 * Implementation of an advanced Run-Length-Encoding.
 * All charactervalues above the limit are seen as escapechar.
 * The value decreased by the limit is the count of the repetition.
 *
 * @author Steffen Schulze
 * @version $Id$
 */
class Rle2Codec : public CompressorCodec
{
	public:
		Rle2Codec()
		{
			#ifdef TEXTOUTPUT
			textOutputMode = 1;
			#endif
		}
	
	
		/**
		 * Encode-method
		 *
		 * Implements the virtual method encode of the baseclass CompressionCodec
		 * @param inputBuffer is pointer to inputdata
		 * @param inputBufferSize is lenght of inputBuffer
		 * @param outputBuffer is pointer to memory for outputdata
		 * @param outputBufferSize is maximal lenght of outputdata
		 * @return length of outputBuffer
		 */
		virtual	UINTMAX encode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize);
		
		/**
		 * Decode-method
		 *
		 * Implements the virtual method decode of the baseclass CompressionCodec
		 * @param inputBuffer is pointer to inputdata
		 * @param inputBufferSize is lenght of inputBuffer
		 * @param outputBuffer is pointer to memory for outputdata
		 * @param outputBufferSize is maximal lenght of outputdata
		 * @return length of outputBuffer
		 */
		virtual	UINTMAX decode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize);
	
	protected:
		/**
		 * The limitvalue for escapesequence
		 */
		enum {ESCAPECHAR = 220};

		/**
		 * The maximal length of repetitions
		 */
		enum {MAXCHARCOUNTS = (255 - ESCAPECHAR)};
		
		#ifdef TEXTOUTPUT
		int textOutputMode;
		#endif

};

#endif
