#include "compress/bwt2/Bwt2Codec.h"
#include "compress/libs/mergeSort.h"

BYTE* Bwt2Codec::bwt_inputBuffer = 0;
UINTMAX Bwt2Codec::bwt_inputBufferSize = 0;

int Bwt2Codec::compareFunction(UINTMAX li, UINTMAX ri)
{
	/**
	 * Dient all Abbruchbedingung fuer den Vergleich.
	 *
	 * Hat die Groesse einer Reihe. Bei jedem Schritt wird sie um 1
	 * incrementiert. Ist die Variable 0 sind die beiden Reihen gleich.
	 */
	UINTMAX ac = bwt_inputBufferSize;
	
	/**
	 * Solange die beiden Positionen in den Reihen sich nicht unterscheiden.
	 */
	while (bwt_inputBuffer[li] == bwt_inputBuffer[ri])
	{
		if (++li == bwt_inputBufferSize)
		{
			li = 0;
		}
		
		if (++ri == bwt_inputBufferSize)
		{
			ri = 0;
		}
		
		if (!--ac)
		{
			return 0;
		}
	}
	
	if (bwt_inputBuffer[li] > bwt_inputBuffer[ri])
	{
		return 1;
	}
	else
	{
		return -1;
	}
}



UINTMAX	Bwt2Codec::encode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize)
{
	if(inputBufferSize > blockSize)
	{
		#ifdef TEXTDEBUG
		isSplitted = true;
		fprintf(stderr, "\t\tSTART encodeBWT2(%d)\n",inputBufferSize);
		fprintf(stderr, "\t\t\tprimaryIndexSize(%d)\n",lengthSize);
		#endif
		
		UINTMAX inputBufferOffset = 0;
		UINTMAX outputBufferOffset = 0;
		UINTMAX bwtBlockSize = blockSize + lengthSize;
		for(; (inputBufferOffset + blockSize) <= inputBufferSize; inputBufferOffset += blockSize)
		{
			encode(
					(inputBuffer + inputBufferOffset) ,
					blockSize ,
					(outputBuffer + outputBufferOffset) ,
					bwtBlockSize
				  );
			outputBufferOffset += bwtBlockSize;
		}
		
		if(inputBufferOffset < inputBufferSize)
		{
			encode(
					(inputBuffer + inputBufferOffset) ,
					(inputBufferSize - inputBufferOffset) ,
					(outputBuffer + outputBufferOffset) ,
					(inputBufferSize - inputBufferOffset + lengthSize)
				  );
		}
		
		UINTMAX outputBufferPos = (outputBufferOffset + (inputBufferSize - inputBufferOffset + lengthSize));
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\tFINISH encodeBWT2(%d)\n",outputBufferPos);
		#endif

		return outputBufferPos;
	}
	else
	{
		#ifdef TEXTDEBUG
		if(isSplitted)
		{
			fprintf(stderr, "\t\t\tSTART encodeBWT2(%d)\n",inputBufferSize);
		}
		else
		{
			fprintf(stderr, "\t\tSTART encodeBWT2(%d)\n",inputBufferSize);
		}
		#endif
	
		UINTMAX indices[inputBufferSize];
		
		
		/**
		* Indexwerte werden jeweils auf die entsprechende Zeichenposition des Eingabepuffer
		* intialisiert. Damit entsprechen sie einer Spalte in der Matrix der BWT
		*/
		for(UINTMAX inputBufferPos = 0; inputBufferPos < inputBufferSize; inputBufferPos++)
		{
			indices[inputBufferPos] = inputBufferPos;
		}
			
		
		
		
		/**
		* Sortierung der Indexwerte, wobei zuerst den Puffervariablen die entsprechenden Werte zugewiesen
		* werden, damit die Vergleichfunktion auf diese zugreifen kann. Danach wir die Sortier-Templatefunktion
		* gestartet.
		*/
		bwt_inputBuffer = inputBuffer;
		bwt_inputBufferSize = inputBufferSize;
		
		mergeSort<UINTMAX>(indices, inputBufferSize, Bwt2Codec::compareFunction);
			
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\tSortierte Reihe in der BWT2-Matrix\n");
		for(UINTMAX inputBufferPos = 0; inputBufferPos < inputBufferSize; inputBufferPos++)
		{
			fprintf(stderr, "\t\t\t(%d):%c:\n",inputBufferPos,inputBuffer[indices[inputBufferPos]]);
		}
		#endif
		
		
		
		/**
		* Aus den sortierten Indexwerten werden nun die Zeichen eine Position vor dem Indexwert in den
		* Ausgabepuffer geschrieben. Diese Zeichen entsprechen der gegenueberliegenden Spalte in der
		* BWT-Matrix.
		*/
		for(UINTMAX inputBufferPos = 0; inputBufferPos < inputBufferSize; inputBufferPos++)
		{
			outputBuffer[inputBufferPos] = inputBuffer[ (indices[inputBufferPos] + inputBufferSize - 1) % inputBufferSize];
		}
		
		
		
		
		/**
		* Primary Index wird ermittelt. Das ist der Index der auf das erste Zeichen des
		* Eingabepuffers zeigt.
		*/
		UINTMAX primaryIndex;
		for(primaryIndex = 0; indices[primaryIndex] != 1; primaryIndex++);
			
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\tprimaryIndex(%d)\n",primaryIndex);
		#endif
	
		
		
		
		/**
		* Primary Index wird geschrieben. Je nach Laenge des Eingabepuffers braucht man hier mehr
		* oder weniger Byte
		*/
		UINTMAX outputBufferPos = inputBufferSize;
		for(BYTE i = 0; i < lengthSize; i++)
		{
			outputBuffer[outputBufferPos++] = BYTE(primaryIndex & 0xFF);
			primaryIndex >>= 8;
		}
	
	
		#ifdef TEXTDEBUG
		if(isSplitted)
		{
			fprintf(stderr, "\t\t\tFINISH encodeBWT2(%d)\n",outputBufferPos);
		}
		else
		{
			fprintf(stderr, "\t\tFINISH encodeBWT2(%d)\n",outputBufferPos);
		}
		#endif
		
		return outputBufferPos;
	}
}




UINTMAX Bwt2Codec::decode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize)
{
	if(inputBufferSize > (blockSize + lengthSize))
	{
		#ifdef TEXTDEBUG
		isSplitted = true;
		fprintf(stderr, "\t\tSTART decodeBWT2(%d)\n",inputBufferSize);
		#endif
		
		UINTMAX inputBufferOffset = 0;
		UINTMAX outputBufferOffset = 0;
		UINTMAX bwtBlockSize = blockSize + lengthSize;
		for(; (inputBufferOffset + bwtBlockSize) <= inputBufferSize; inputBufferOffset += bwtBlockSize)
		{
			decode(
					(inputBuffer + inputBufferOffset) ,
					bwtBlockSize ,
					(outputBuffer + outputBufferOffset) ,
					blockSize
				  );
			outputBufferOffset += blockSize;
		}
		
		if(inputBufferOffset < inputBufferSize)
		{
			decode(
					(inputBuffer + inputBufferOffset) ,
					(inputBufferSize - inputBufferOffset) ,
					(outputBuffer + outputBufferOffset) ,
					(inputBufferSize - inputBufferOffset - lengthSize)
				  );
		}
		
		UINTMAX outputBufferPos = (outputBufferOffset + (inputBufferSize - inputBufferOffset - lengthSize));
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\tFINISH decodeBWT2(%d)\n",outputBufferPos);
		#endif

		return outputBufferPos;
	}
	else
	{
		#ifdef TEXTDEBUG
		if(isSplitted)
		{
			fprintf(stderr, "\t\t\tSTART decodeBWT2(%d)\n",inputBufferSize);
		}
		else
		{
			fprintf(stderr, "\t\tSTART decodeBWT2(%d)\n",inputBufferSize);
		}
		#endif
		
		/**
		* Puffer fuer das Encodieren des Eingangspuffers
		*/
		BYTE F[inputBufferSize];
		
		/**
		* Haeufigkeit der Zeichen im Eingangspuffer
		*/
		UINTMAX charCounts[256];
		
		/**
		* Indexwerte des Eingangspuffers
		*/
		UINTMAX indices[inputBufferSize];
	
		/**
		* Index der auf Beginn der Nachricht zeigt
		*/
		UINTMAX primaryIndex = 0;
		
		
		/**
		* Primary Index wird gelesen. Je nach Laenge des Eingabepuffers braucht man hier mehr
		* oder weniger Byte
		* In Scope gefasst um Compiler Optimierungen zu vereinfachen
		*/
		{
			UINTMAX inputBufferPos = inputBufferSize - 1;
			for(BYTE i = 0; i < lengthSize; i++)
			{
				inputBufferSize--;
				primaryIndex <<= 8;
				primaryIndex |= inputBuffer[inputBufferPos--];
				#ifdef TEXTDEBUG
				fprintf(stderr, "\t\t\tpi(%d) ibs(%d)\n",primaryIndex, inputBufferSize);
				#endif
			}
		}
		
		
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\tprimaryIndex(%d)\n",primaryIndex);
		#endif
		
		/**
		* Zeichenzaehler wird initialisiert
		*/
		for (UINT16 i = 0; i < 256; i++)
		{
			charCounts[i] = 0;
		}
		
		/**
		* Zeichenhaeufigkeit im Eingabepuffer werden gezaehlt
		*/
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\tinputBuffer\n");
		#endif
		for (UINTMAX inputBufferPos = 0; inputBufferPos < inputBufferSize; inputBufferPos++)
		{
			charCounts[inputBuffer[inputBufferPos]] ++;
				#ifdef TEXTDEBUG
				fprintf(stderr, "\t\t\t\t(%d):%c:\n",inputBufferPos,inputBuffer[inputBufferPos]);
				#endif
		}
		
		/**
		* Die Zeichen des Eingabepuffer werden sortiert hintereinandern
		* geschrieben
		*/
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\tsortetCol\n");
		#endif
		for (UINTMAX i = 0, inputBufferPos = 0; i < 256; i++)
		{
			for (UINTMAX j = 0; j < charCounts[i]; j++)
			{
				F[inputBufferPos++] = i;
				#ifdef TEXTDEBUG
				fprintf(stderr, "\t\t\t\t(%d):%c:\n",inputBufferPos,i);
				#endif
			}
		}
		
		/**
		* k sollte an dieser Stelle gleich inputBufferSize sein,
		* ansonsten ist was schiefgelaufen
		*/
		
		for (UINTMAX i = 0, inputBufferPos = 0; i < 256; i++)
		{
			while( (i > F[inputBufferPos]) && (inputBufferPos < inputBufferSize) )
			{
				inputBufferPos++;
			}
			
			/**
			* it will get fake values if there is no i in F, but
			* that won't bring us any problems
			*/
			charCounts[i] = inputBufferPos;
		}
		
		for(UINTMAX inputBufferPos = 0; inputBufferPos < inputBufferSize; inputBufferPos++)
		{
			indices[charCounts[inputBuffer[inputBufferPos]]++] = inputBufferPos;
		}
		
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\toutputBuffer\n");
		#endif
		for(UINTMAX inputBufferPos = 0, j = primaryIndex; inputBufferPos < inputBufferSize; inputBufferPos++)
		{
			outputBuffer[inputBufferPos]	= inputBuffer[j];
			#ifdef TEXTDEBUG
			fprintf(stderr, "\t\t\t\t(%d):%c:\n",inputBufferPos,inputBuffer[j]);
			#endif
			j								= indices[j];
		}
		
		
		UINTMAX outputBufferPos	= inputBufferSize;
		
		#ifdef TEXTDEBUG
		if(isSplitted)
		{
			fprintf(stderr, "\t\t\tFINISH decodeBWT2(%d)\n",outputBufferPos);
		}
		else
		{
			fprintf(stderr, "\t\tFINISH decodeBWT2(%d)\n",outputBufferPos);
		}
		#endif
		
		return outputBufferPos;
	}
}


