#include "compress/huffman/HuffmanCodec.h"
#include "compress/libs/BitField.h"

#include "compress/libs/charCount.h"


UINT16	HuffmanCodec::encode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize)
{
	
	#ifdef TEXTOUTPUT
	if( textOutputMode )
	{
		fprintf(stderr, "\t\tSTART encodeHuffman(%d)\n",inputBufferSize);
	}
	#endif
	
	///////////////////////////////////////////////////////////////////////////
	//zaehle haeufigkeiten
	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\tSTART charCount()\n");
	#endif
	
	/**
	 * Zaehlt die Haeufigkeit eines Zeichens im Eingabepuffer
	 */
	UINTMAX charCounter[256];
	
	/**
	 * Der Sauberkeit halber
	 */
	for(UINT16 i = 0; i < 256; i++)
	{
		charCounter[i] = 0;
	}
	
	/**
	 * alle Zeichen im Eingabepuffer werden gezaehlt
	 */
	for(UINTMAX inputBufferPos = 0; inputBufferPos < inputBufferSize; inputBufferPos++)
	{
		/**
		 * Diese Funktion schuetzt vor Variablenueberlaeufen. Kann also auch durch ein
		 * einfaches Inkrement ersetzt werden.
		 */
		charCount(inputBuffer[inputBufferPos],charCounter);
	}
	
	BYTE charIndex[256];
	UINT16 charIndexCount = charCountFinish(charCounter, charIndex);
	//
	///////////////////////////////////////////////////////////////////////////
	
	///////////////////////////////////////////////////////////////////////////
	//erstelle charCodes
	CharCode charCodes[charIndexCount];
	
	findHuffmanTree(charCounter, charIndex, charIndexCount, charCodes);
	buildHuffmanTree(charCodes, charIndexCount);
	#ifdef TEXTDEBUG
	printCharCodes(charCodes, charIndexCount);
	#endif
	//
	///////////////////////////////////////////////////////////////////////////
	
	
	
	///////////////////////////////////////////////////////////////////////////
	//encode daten
	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\tSTART encoding\n");
	#endif
	
	///////////////////////////////
	// write huffmanntree
	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\t\twrite Huffmantree\n");
	#endif
	
	UINTMAX outputBufferPos = 0;
	
	UINT16 currCharCode = 0;
	UINT16 currBitCount = 0;
	UINT16 lastIndex	= 0;
	outputBuffer[lastIndex] = 0;
	
	while(currCharCode < charIndexCount)
	{
		if(currBitCount != charCodes[currCharCode].bitcount)
		{
			#ifdef TEXTDEBUG
			fprintf(stderr, "\n\t\t\t\t:i-%d: %d", outputBuffer[lastIndex],currBitCount);
			#endif

			while (currBitCount < charCodes[currCharCode].bitcount)
			{
				outputBuffer[outputBufferPos] = 0;
				outputBufferPos++;
				currBitCount++;
				#ifdef TEXTDEBUG
				fprintf(stderr, "\n\t\t\t\t:i-0: %d",currBitCount);
				#endif
			}
	
			lastIndex = outputBufferPos - 1;
		}	
		
		#ifdef TEXTDEBUG
		fprintf(stderr, " :%c:",charCodes[currCharCode].index);
		#endif

		outputBuffer[outputBufferPos] = charCodes[currCharCode].index;
		outputBufferPos++;
		outputBuffer[lastIndex]++;
		
		currCharCode++;
	}
	// write huffmanntree
	///////////////////////////////
	
	
	///////////////////////////////
	// write bitcode
	#ifdef TEXTDEBUG
	fprintf(stderr, "\n\t\t\t\twrite bitcode\n");
	//printCharCodes(charCodes, charIndexCount);
	#endif
	//alle zeichen werden codiert
	
	BitField bitFieldOutputBuffer(outputBuffer + outputBufferPos);
	bitFieldOutputBuffer.reset();
	
	for(UINTMAX inputBufferPos = 0; inputBufferPos < inputBufferSize; inputBufferPos++)
	{
		UINT16	currCharCode;
		BYTE	currChar = inputBuffer[inputBufferPos];
		
		//suche passenden code in charCodes
		for(currCharCode = 0; (charCodes[currCharCode].index != currChar);currCharCode++);
		
		UINTMAX	bitCode	= charCodes[currCharCode].bitcode;
		UINT16	bitCount= charCodes[currCharCode].bitcount;
		
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\t\t\t:%x:%d:\n",bitCode, bitCount);
		#endif
		
		bitFieldOutputBuffer.writeBits(bitCode, bitCount);
		
		MEMORYCHECK((bitFieldOutputBuffer.getPos() + 10 + outputBufferPos) > outputBufferSize)
	
	}
	
	outputBufferPos = bitFieldOutputBuffer.getPos() + outputBufferPos;
	outputBufferPos++;
	outputBuffer[outputBufferPos]	=	bitFieldOutputBuffer.getBitPos();
	outputBufferPos++;
	// write bitcode
	///////////////////////////////
		
	//
	///////////////////////////////////////////////////////////////////////////
	
	
	
	
	#ifdef TEXTOUTPUT
	if( textOutputMode )
	{
		fprintf(stderr, "\t\tFINISH encodeHuffman(%d)\n", outputBufferPos);
	}
	#endif
	
	return outputBufferPos;
}




UINT16 HuffmanCodec::decode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize)
{
	UINTMAX outputBufferPos	= 0;
	UINTMAX inputBufferPos	= 0;
	
	#ifdef TEXTOUTPUT
	if( textOutputMode )
	{
		fprintf(stderr, "\t\tSTART decodeHuffman(%d)\n",inputBufferSize);
	}
	#endif
	
	///////////////////////////////
	// read huffmanntree
	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\tread huffmanntree\n");
	#endif
	
	UINT16 charIndexCount = 0;
	CharCode charCodes[256];
	
	{
		UINT16 currNodeCount	= 0;
		UINT16 currCharCode		= 0;
		UINT16 currBitcount 	= 1;
		UINT16 openBranches		= 1;
		while(openBranches)
		{
			openBranches <<= 1; // *2
			
			currNodeCount = inputBuffer[inputBufferPos];
			inputBufferPos++;
			
			openBranches -= currNodeCount;
			
			for(UINT16 i = 0; i < currNodeCount; i++)
			{
				charCodes[currCharCode].index = inputBuffer[inputBufferPos];
				charCodes[currCharCode].bitcount = currBitcount;
				
				currCharCode++;
				inputBufferPos++;
			}
			
			currBitcount++;
			
		}
		charIndexCount = currCharCode;
	}
	
	buildHuffmanTree(charCodes, charIndexCount);
	#ifdef TEXTDEBUG
	printCharCodes(charCodes, charIndexCount);
	#endif
	// read huffmanntree
	///////////////////////////////
	
	///////////////////////////////
	// construct decodetree
	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\tconstruct decodetree\n");
	#endif
	
	CharNode charNodes[1024];
	for(UINT16 i = 0; i < 1024; i++)
	{
		charNodes[i].child[0] = 0;
		charNodes[i].child[1] = 0;
	}
	
	UINT16 nextFreeNode = 1;
	
	for(UINT16 currCharCode = 0; currCharCode < charIndexCount; currCharCode++)
	{
		UINT16 currNode = 0;
		UINTMAX currBitcode = charCodes[currCharCode].bitcode;
		for(UINT16 i = charCodes[currCharCode].bitcount; i > 0 ; i--)
		{
			BYTE currBit = (currBitcode >> (i-1)) & 1;
			if(charNodes[currNode].child[currBit] == 0)
			{
				charNodes[currNode].child[currBit] = nextFreeNode;
				currNode = nextFreeNode;
				nextFreeNode++;
			}
			else
			{
				currNode = charNodes[currNode].child[currBit];
			}
	
		}
		// Blatt
		charNodes[currNode].child[0] = charCodes[currCharCode].index;
	}
	
	#ifdef TEXTDEBUG
	for(UINT16 i = 0; i < nextFreeNode; i++)
	{
		if(charNodes[i].child[1])
		{
			fprintf(stderr, "\t\t\t\tNODE[%d] :%d: :%d:\n",i ,charNodes[i].child[0] ,charNodes[i].child[1]);
		}
		else
		{
			fprintf(stderr, "\t\t\t\tNODE[%d] :%c:\n",i ,charNodes[i].child[0]);
		}
	}	
	#endif
	// construct decodetree
	///////////////////////////////
	
	///////////////////////////////
	// decode bitcode
	BitField bitFieldInputBuffer(inputBuffer + inputBufferPos);
	
	UINT16	currNode			= 0;
	UINTMAX bitFieldLength		= inputBufferSize - inputBufferPos - 2;
	BYTE	bitFieldTailLength	= inputBuffer[inputBufferSize - 1];
	
	while
	(
		!(
			(bitFieldInputBuffer.getPos() == bitFieldLength)
			&&
			(bitFieldInputBuffer.getBitPos() == bitFieldTailLength)
		 )
	)
	{
		BYTE currBit = bitFieldInputBuffer.readBit();
		
		#ifdef TEXTDEBUG
		fprintf(stderr, "%d",currBit);
		#endif

		currNode = charNodes[currNode].child[currBit];
		if(charNodes[currNode].child[1] == 0)
		{
			outputBuffer[outputBufferPos] = charNodes[currNode].child[0];
			#ifdef TEXTDEBUG
			fprintf(stderr, "\t\t\t\tout[%d] :%c:\n",outputBufferPos ,charNodes[currNode].child[0]);
			#endif
			outputBufferPos++;
			currNode = 0;
		}
		
		MEMORYCHECK((outputBufferPos + 10) > outputBufferSize)

	}
	// decode bitcode
	///////////////////////////////
	
	
	#ifdef TEXTOUTPUT
	if( textOutputMode )
	{
		fprintf(stderr, "\t\tFINISH decodeHuffman(%d)\n",outputBufferPos);
	}
	#endif
	
	return outputBufferPos;
}
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////













///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// findHuffmanTree

//////////////////////////////
//
void 	HuffmanCodec::findHuffmanTree(UINT16 charCounter[], BYTE* charIndex, UINT16 charIndexCount, CharCode* charCodes)
{
	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\tSTART findHuffmanTree\n");
	#endif
	
	CharCodeNode charCodeNodes[charIndexCount];
	
	//initialisiere das feld, fuer die codegenerierung
	for(UINT16 i = 0; i < charIndexCount; i++)
	{
		charCodeNodes[i].count			= charCounter[charIndex[i]];
		charCodeNodes[i].code.index		= charIndex[i];
		charCodeNodes[i].code.bitcount	= 0;
		charCodeNodes[i].code.bitcode	= 0;
		charCodeNodes[i].buddy			= i;
	}
	
	bool ready = FALSE;
	while(!ready)
	{
		//hole ersten zweig
		UINT16 branch1 = findTopBuddy( (charIndexCount - 1), charCodeNodes);
	
		if(branch1 > 0)
		{
			//hole zweiten zweig
			UINT16 branch2 = findTopBuddy( (branch1 - 1), charCodeNodes);
			
			//variablenueberlaufschutz
			if(
				(charCodeNodes[branch1].count > (MAX_UINTMAX >> 1))
				||
				(charCodeNodes[branch2].count > (MAX_UINTMAX >> 1))
			  )
			{
				for(UINT16 i = 0; i < charIndexCount; i++)
				{
					if(charCodeNodes[i].count > 1)
					{
						charCodeNodes[i].count >>= 1;
					}
				}
				#ifdef TEXTDEBUG
				fprintf(stderr, "\t\t\t\tVARIABLENUEBERLAUFSCHUTZ\n");
				printCharCodeNodes(charCodeNodes, charIndexCount);
				#endif
			}
			
			mergeBranches(branch1, branch2, charCodeNodes, charIndexCount);
			sortTree(charCodeNodes, charIndexCount);
		}
		else
		{
			ready = TRUE;
		}
	}

	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\t\tstelle charCodes fertig\n");
	#endif
	//initialisiere das feld, fuer die codegenerierung
	for(UINT16 i = 0; i < charIndexCount; i++)
	{
		charCodes[i] = charCodeNodes[i].code;
	}
	
	//bubblesort
	ready = FALSE;
	CharCode charCodeCache;
	while(!ready)
	{
		ready = TRUE;
		for(UINT16 i = 1; i < charIndexCount; i++)
		{
			if(charCodes[i - 1].bitcount > charCodes[i].bitcount)
			{
				charCodeCache		= charCodes[i - 1];
				charCodes[i - 1]	= charCodes[i];
				charCodes[i]		= charCodeCache;
				ready = FALSE;
			}
		}
	}

	

	#ifdef TEXTDEBUG
	fprintf(stderr, "\n\t\t\tFINISH findHuffmanTree()\n");
	#endif
}





	//////////////////////////////
	//
	UINT16	HuffmanCodec::findTopBuddy(UINT16 start, CharCodeNode* charCodeNodes)
	{
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\t\tSTART findTopBuddy(%d)\n", start);
		#endif
		
		//////////////////////////////
		// finde obersten branchknoten in der liste der buddies
		UINT16 startBuddy				= start;
		UINT16 firstBuddy				= start;
		CharCodeNode currCharCodeNode	= charCodeNodes[start];
		
		while(currCharCodeNode.buddy != startBuddy)
		{
			if(firstBuddy > currCharCodeNode.buddy)
			{
				firstBuddy = currCharCodeNode.buddy;
			}
			currCharCodeNode = charCodeNodes[currCharCodeNode.buddy];
		}
		// finde obersten branchknoten in der liste der buddies
		//////////////////////////////
		
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\t\tFINISH findTopBuddy(%d(%d) width %d)\n", firstBuddy, charCodeNodes[firstBuddy].code.index, charCodeNodes[firstBuddy].count);
		#endif
		
		return firstBuddy;
	}
	
	
	//////////////////////////////
	//
	void	HuffmanCodec::mergeBranches(UINT16 branch1, UINT16 branch2, CharCodeNode* charCodeNodes, UINT16 charIndexCount)
	{
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\t\tSTART mergeBranches(%d(%d):%d:, %d(%d):%d:)\n", branch1, charCodeNodes[branch1].code.index, charCodeNodes[branch1].count,  branch2, charCodeNodes[branch2].code.index, charCodeNodes[branch2].count);
		#endif
		
		UINTMAX newCount = charCodeNodes[branch1].count + charCodeNodes[branch2].count;
		
		tagBranch(branch1, newCount, charCodeNodes);
		tagBranch(branch2, newCount, charCodeNodes);
		
		UINT16 cache = charCodeNodes[branch1].buddy;
		charCodeNodes[branch1].buddy = charCodeNodes[branch2].buddy;
		charCodeNodes[branch2].buddy = cache;
		
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\t\tFINISH mergeBranches()\n");
		#endif
	}
	
	
	
	
		//////////////////////////////
		//
		void	HuffmanCodec::tagBranch(UINT16 branch, UINTMAX count, CharCodeNode* charCodeNodes)
		{
			#ifdef TEXTDEBUG
			fprintf(stderr, "\t\t\t\t\tSTART tagBranch(%d(%d) with %d)\n",branch, charCodeNodes[branch].code.index, count);
			#endif
			//fuer alle knoten/buddies im zweig
			UINT16 startBuddy				= branch;
			CharCodeNode* currCharCodeNode	= &(charCodeNodes[branch]);
		
			bool first = TRUE;
			while(((*currCharCodeNode).buddy != charCodeNodes[startBuddy].buddy) || first)
			{
				#ifdef TEXTDEBUG
				fprintf(stderr, "\t\t\t\t\t\t(%d)\n",(*currCharCodeNode).code.index);
				#endif
				(*currCharCodeNode).count			=	count;
				(*currCharCodeNode).code.bitcount++;
				
				currCharCodeNode = &(charCodeNodes[(*currCharCodeNode).buddy]);
				first = FALSE;
			};

			#ifdef TEXTDEBUG
			fprintf(stderr, "\t\t\t\t\tFINISH tagBranch()\n");
			#endif
		}

	
		
		
		
		
	///////////////////////////////////////////////////////////////////////////
	
	//////////////////////////////
	//
		void	HuffmanCodec::sortTree(CharCodeNode* charCodeNodes, UINT16 charIndexCount)
	{
		#ifdef TEXTDEBUG
		fprintf(stderr, "\t\t\t\tSTART sortTree()\n");
		#endif
		//bubblesort
		
		//alles andere als optimal
		//da jedesmal zum index und zurueck abgebildet werden muss
		
		//alle buddies auf index abbilden
		for(UINT16 i = 0; i < charIndexCount; i++)
		{
			charCodeNodes[i].buddy	= charCodeNodes[charCodeNodes[i].buddy].code.index;
		}
		
		
		bool ready = FALSE;
		CharCodeNode charCodeNodeCache;
		while(!ready)
		{
			ready = TRUE;
			for(UINT16 i = 1; i < charIndexCount; i++)
			{
				if(charCodeNodes[i - 1].count < charCodeNodes[i].count)
				{
					charCodeNodeCache		= charCodeNodes[i - 1];
					charCodeNodes[i - 1]	= charCodeNodes[i];
					charCodeNodes[i]		= charCodeNodeCache;
					ready = FALSE;
				}
			}
		}
	
		//alle buddies vom index herleiten
		for(UINT16 i = 0; i < charIndexCount; i++)
		{
			UINT16 j;
			for(j = 0; charCodeNodes[i].buddy != charCodeNodes[j].code.index; j++);
			charCodeNodes[i].buddy = j;
		
		}
	
	
	}
		
		
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

	
	
	
	
	
	
	
	
	
	
	
	
	
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// buildHuffmanTree
	void	HuffmanCodec::buildHuffmanTree(CharCode* charCodes, UINT16 charIndexCount)
{
	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\tSTART buildHuffmanTree\n");
	for(UINT16 i = 0; i < charIndexCount; i++)
	{
		charCodes[i].bitcode = 0;
	}
	#endif
	
	bool	ready			= FALSE;
	UINT16	currCharCode	= 0;
	UINT16	bitcount		= 1;
	UINTMAX	bitcode			= 0;
	while(!ready)
	{
		
		if( !(currCharCode < charIndexCount) )
		{
			#ifdef TEXTDEBUG
			fprintf(stderr, "\nPANIC in buildHuffmanTree - input is not a clean Tree!\n");
			#endif
			while(1);
		}
		
		
		for(UINT16 level = bitcount; level < charCodes[currCharCode].bitcount; level++)
		{
			bitcount	++;
			bitcode		<<=	1;
		}
		
		//SETZE CODE
		charCodes[currCharCode].bitcode = bitcode;
		
		if( (bitcode & 1) == 0)
		{
			//kleiner sprung
			bitcode |= 1; 
		}
		else
		{
			//grosser sprung;
			while(
					( (bitcode & 1) != 0 )
					&&
					(!ready)
				 )
			{
				bitcount--;
				bitcode	>>= 1;
				
				if(bitcount == 0)
				{
					ready = TRUE;
				}
			}
			
			if(!ready)
			{
				bitcode |= 1; 
			}
				
		}
		
		currCharCode++;
	}
	
	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\tFINISH buildHuffmanTree\n");
	#endif
}

//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
	



//////////////////////////
// Debugausgabe
#ifdef TEXTDEBUG
void	HuffmanCodec::printCharCodes(CharCode* charCodes, UINT16 charCodeCount)
{	
	fprintf(stderr, "\n");
	for(UINT16 i = 0; i < charCodeCount; i++)
	{
		fprintf(stderr, "\t\t\tcharCodes[%d] = bitcount (%d) bitcode(%X) :%d:%c:\n",i, charCodes[i].bitcount, charCodes[i].bitcode, charCodes[i].index, charCodes[i].index);
	}
	fprintf(stderr, "\n");
}

void	HuffmanCodec::printCharCodeNodes(CharCodeNode* charCodeNodes, UINT16 charIndexCount)
{
	fprintf(stderr, "\n");
	for(UINT16 i = 0; i < charIndexCount; i++)
	{
		fprintf(stderr, "\t\t\tcharCodeNodes[%d] = index (%d) count(%d) bitcount(%d) bitcode(%X) buddy(%d) :%c:\n",i, charCodeNodes[i].code.index ,charCodeNodes[i].count, charCodeNodes[i].code.bitcount, charCodeNodes[i].code.bitcode, charCodeNodes[i].buddy, charCodeNodes[i].code.index);
	}
	fprintf(stderr, "\n");
}
#endif


